# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
# pylint: disable=W0201

from ..protobufs import main_pb2
from ..firmware_version import FirmwareVersion


class DeviceIdentity:
    """
    Representation of data gathered during device identification.
    """

    @property
    def device_id(self) -> int:
        """
        Unique ID of the device hardware.
        """

        return self._device_id

    @device_id.setter
    def device_id(self, value: int) -> None:
        self._device_id = value

    @property
    def serial_number(self) -> int:
        """
        Serial number of the device.
        """

        return self._serial_number

    @serial_number.setter
    def serial_number(self, value: int) -> None:
        self._serial_number = value

    @property
    def name(self) -> str:
        """
        Name of the product.
        """

        return self._name

    @name.setter
    def name(self, value: str) -> None:
        self._name = value

    @property
    def axis_count(self) -> int:
        """
        Number of axes this device has.
        """

        return self._axis_count

    @axis_count.setter
    def axis_count(self, value: int) -> None:
        self._axis_count = value

    @property
    def firmware_version(self) -> FirmwareVersion:
        """
        Version of the firmware.
        """

        return self._firmware_version

    @firmware_version.setter
    def firmware_version(self, value: FirmwareVersion) -> None:
        self._firmware_version = value

    def __repr__(self) -> str:
        return str(self.__dict__)

    @staticmethod
    def from_protobuf(pb_data: main_pb2.DeviceIdentity) -> 'DeviceIdentity':
        instance = DeviceIdentity()
        instance.device_id = pb_data.device_id
        instance.serial_number = pb_data.serial_number
        instance.name = pb_data.name
        instance.axis_count = pb_data.axis_count
        instance.firmware_version = FirmwareVersion.from_protobuf(pb_data.firmware_version)
        return instance



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

from typing import TYPE_CHECKING, List
from ..call import call, call_async
from .device_io_info import DeviceIOInfo

from ..protobufs import main_pb2

if TYPE_CHECKING:
    from .device import Device


class DeviceIO:
    """
    Class providing access to the I/O channels of the device.
    """

    def __init__(self, device: 'Device'):
        self._device = device

    def get_all_digital_inputs(
            self
    ) -> List[bool]:
        """
        Returns the current values of all digital input channels.

        Returns:
            True if voltage is present on the input channel and false otherwise.
        """
        request = main_pb2.DeviceGetAllDigitalIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "di"
        response = main_pb2.DeviceGetAllDigitalIOResponse()
        call("device/get_all_digital_io", request, response)
        return list(response.values)

    async def get_all_digital_inputs_async(
            self
    ) -> List[bool]:
        """
        Returns the current values of all digital input channels.

        Returns:
            True if voltage is present on the input channel and false otherwise.
        """
        request = main_pb2.DeviceGetAllDigitalIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "di"
        response = main_pb2.DeviceGetAllDigitalIOResponse()
        await call_async("device/get_all_digital_io", request, response)
        return list(response.values)

    def get_all_digital_outputs(
            self
    ) -> List[bool]:
        """
        Returns the current values of all digital output channels.

        Returns:
            True if the output channel is conducting and false otherwise.
        """
        request = main_pb2.DeviceGetAllDigitalIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "do"
        response = main_pb2.DeviceGetAllDigitalIOResponse()
        call("device/get_all_digital_io", request, response)
        return list(response.values)

    async def get_all_digital_outputs_async(
            self
    ) -> List[bool]:
        """
        Returns the current values of all digital output channels.

        Returns:
            True if the output channel is conducting and false otherwise.
        """
        request = main_pb2.DeviceGetAllDigitalIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "do"
        response = main_pb2.DeviceGetAllDigitalIOResponse()
        await call_async("device/get_all_digital_io", request, response)
        return list(response.values)

    def get_all_analog_inputs(
            self
    ) -> List[float]:
        """
        Returns the current values of all analog input channels.

        Returns:
             Measurements of the voltage present on the input channels.
        """
        request = main_pb2.DeviceGetAllAnalogIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "ai"
        response = main_pb2.DeviceGetAllAnalogIOResponse()
        call("device/get_all_analog_io", request, response)
        return list(response.values)

    async def get_all_analog_inputs_async(
            self
    ) -> List[float]:
        """
        Returns the current values of all analog input channels.

        Returns:
             Measurements of the voltage present on the input channels.
        """
        request = main_pb2.DeviceGetAllAnalogIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "ai"
        response = main_pb2.DeviceGetAllAnalogIOResponse()
        await call_async("device/get_all_analog_io", request, response)
        return list(response.values)

    def get_all_analog_outputs(
            self
    ) -> List[float]:
        """
        Returns the current values of all analog output channels.

        Returns:
             Measurements of voltage that the output channels are conducting.
        """
        request = main_pb2.DeviceGetAllAnalogIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "ao"
        response = main_pb2.DeviceGetAllAnalogIOResponse()
        call("device/get_all_analog_io", request, response)
        return list(response.values)

    async def get_all_analog_outputs_async(
            self
    ) -> List[float]:
        """
        Returns the current values of all analog output channels.

        Returns:
             Measurements of voltage that the output channels are conducting.
        """
        request = main_pb2.DeviceGetAllAnalogIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "ao"
        response = main_pb2.DeviceGetAllAnalogIOResponse()
        await call_async("device/get_all_analog_io", request, response)
        return list(response.values)

    def get_digital_input(
            self,
            channel_number: int
    ) -> bool:
        """
        Returns the current value of the specified digital input channel.

        Args:
            channel_number: Channel number starting at 1.

        Returns:
            True if voltage is present on the input channel and false otherwise.
        """
        request = main_pb2.DeviceGetDigitalIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "di"
        request.channel_number = channel_number
        response = main_pb2.DeviceGetDigitalIOResponse()
        call("device/get_digital_io", request, response)
        return response.value

    async def get_digital_input_async(
            self,
            channel_number: int
    ) -> bool:
        """
        Returns the current value of the specified digital input channel.

        Args:
            channel_number: Channel number starting at 1.

        Returns:
            True if voltage is present on the input channel and false otherwise.
        """
        request = main_pb2.DeviceGetDigitalIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "di"
        request.channel_number = channel_number
        response = main_pb2.DeviceGetDigitalIOResponse()
        await call_async("device/get_digital_io", request, response)
        return response.value

    def get_digital_output(
            self,
            channel_number: int
    ) -> bool:
        """
        Returns the current value of the specified digital output channel.

        Args:
            channel_number: Channel number starting at 1.

        Returns:
            True if the output channel is conducting and false otherwise.
        """
        request = main_pb2.DeviceGetDigitalIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "do"
        request.channel_number = channel_number
        response = main_pb2.DeviceGetDigitalIOResponse()
        call("device/get_digital_io", request, response)
        return response.value

    async def get_digital_output_async(
            self,
            channel_number: int
    ) -> bool:
        """
        Returns the current value of the specified digital output channel.

        Args:
            channel_number: Channel number starting at 1.

        Returns:
            True if the output channel is conducting and false otherwise.
        """
        request = main_pb2.DeviceGetDigitalIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "do"
        request.channel_number = channel_number
        response = main_pb2.DeviceGetDigitalIOResponse()
        await call_async("device/get_digital_io", request, response)
        return response.value

    def get_analog_input(
            self,
            channel_number: int
    ) -> float:
        """
        Returns the current value of the specified analog input channel.

        Args:
            channel_number: Channel number starting at 1.

        Returns:
             A measurementsof the voltage present on the input channel.
        """
        request = main_pb2.DeviceGetAnalogIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "ai"
        request.channel_number = channel_number
        response = main_pb2.DeviceGetAnalogIOResponse()
        call("device/get_analog_io", request, response)
        return response.value

    async def get_analog_input_async(
            self,
            channel_number: int
    ) -> float:
        """
        Returns the current value of the specified analog input channel.

        Args:
            channel_number: Channel number starting at 1.

        Returns:
             A measurementsof the voltage present on the input channel.
        """
        request = main_pb2.DeviceGetAnalogIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "ai"
        request.channel_number = channel_number
        response = main_pb2.DeviceGetAnalogIOResponse()
        await call_async("device/get_analog_io", request, response)
        return response.value

    def get_analog_output(
            self,
            channel_number: int
    ) -> float:
        """
        Returns the current values of the specified analog output channel.

        Args:
            channel_number: Channel number starting at 1.

        Returns:
            A measurement of voltage that the output channel is conducting.
        """
        request = main_pb2.DeviceGetAnalogIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "ao"
        request.channel_number = channel_number
        response = main_pb2.DeviceGetAnalogIOResponse()
        call("device/get_analog_io", request, response)
        return response.value

    async def get_analog_output_async(
            self,
            channel_number: int
    ) -> float:
        """
        Returns the current values of the specified analog output channel.

        Args:
            channel_number: Channel number starting at 1.

        Returns:
            A measurement of voltage that the output channel is conducting.
        """
        request = main_pb2.DeviceGetAnalogIORequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_type = "ao"
        request.channel_number = channel_number
        response = main_pb2.DeviceGetAnalogIOResponse()
        await call_async("device/get_analog_io", request, response)
        return response.value

    def set_all_digital_outputs(
            self,
            values: List[bool]
    ) -> None:
        """
        Sets values for all digital output channels.

        Args:
            values: True to set the output channel to conducting and false to turn it off.
        """
        request = main_pb2.DeviceSetAllDigitalOutputsRequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.values.extend(values)
        call("device/set_all_digital_outputs", request)

    async def set_all_digital_outputs_async(
            self,
            values: List[bool]
    ) -> None:
        """
        Sets values for all digital output channels.

        Args:
            values: True to set the output channel to conducting and false to turn it off.
        """
        request = main_pb2.DeviceSetAllDigitalOutputsRequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.values.extend(values)
        await call_async("device/set_all_digital_outputs", request)

    def set_digital_output(
            self,
            channel_number: int,
            value: bool
    ) -> None:
        """
        Sets value for the specified digital output channel.

        Args:
            channel_number: Channel number starting at 1.
            value: True to set the output channel to conducting and false to turn it off.
        """
        request = main_pb2.DeviceSetDigitalOutputRequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_number = channel_number
        request.value = value
        call("device/set_digital_output", request)

    async def set_digital_output_async(
            self,
            channel_number: int,
            value: bool
    ) -> None:
        """
        Sets value for the specified digital output channel.

        Args:
            channel_number: Channel number starting at 1.
            value: True to set the output channel to conducting and false to turn it off.
        """
        request = main_pb2.DeviceSetDigitalOutputRequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        request.channel_number = channel_number
        request.value = value
        await call_async("device/set_digital_output", request)

    def get_channels_info(
            self
    ) -> DeviceIOInfo:
        """
        Returns the number of I/O channels the device has.

        Returns:
            An object containing the number of I/O channels the device has.
        """
        request = main_pb2.DeviceGetIOChannelsInfoRequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        response = main_pb2.DeviceIOInfo()
        call("device/get_io_info", request, response)
        return DeviceIOInfo.from_protobuf(response)

    async def get_channels_info_async(
            self
    ) -> DeviceIOInfo:
        """
        Returns the number of I/O channels the device has.

        Returns:
            An object containing the number of I/O channels the device has.
        """
        request = main_pb2.DeviceGetIOChannelsInfoRequest()
        request.interface_id = self._device.connection.interface_id
        request.device = self._device.device_address
        response = main_pb2.DeviceIOInfo()
        await call_async("device/get_io_info", request, response)
        return DeviceIOInfo.from_protobuf(response)



from ctypes import c_void_p, c_int, c_int64, c_uint8, CFUNCTYPE
from ctypes import cdll
from typing import Any
import platform
import sys


def load_library(name):
    return cdll.LoadLibrary(name)


def _load_library() -> Any:
    os_system = platform.system().lower()
    os_machine = platform.machine().lower()
    if os_machine.startswith("arm"):
        arch = "arm"
    else:
        arch = "amd64" if sys.maxsize > 2**32 else "386"
    ext = ""
    if os_system == "linux":
        ext = ".so"
    if os_system == "darwin":
        ext = ".dylib"
    lib_name = "zaber-motion-lib-{}-{}{}".format(os_system, arch, ext)
    return load_library(lib_name)


lib = _load_library()

CALLBACK = CFUNCTYPE(None, c_void_p, c_int64)

c_call = lib.call
c_call.argtypes = [c_void_p, c_int64, CALLBACK, c_uint8]
c_call.restype = c_int

c_set_event_handler = lib.setEventHandler
c_set_event_handler.argtypes = [c_int64, CALLBACK]
c_set_event_handler.restype = None



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
# pylint: disable=W0201

from ..protobufs import main_pb2
from .message_type import MessageType


class Response:
    """
    Response message from the device.
    """

    @property
    def device_address(self) -> int:
        """
        Number of the device that sent the message.
        """

        return self._device_address

    @device_address.setter
    def device_address(self, value: int) -> None:
        self._device_address = value

    @property
    def axis_number(self) -> int:
        """
        Number of the axis which the response applies to. Zero denotes device scope.
        """

        return self._axis_number

    @axis_number.setter
    def axis_number(self, value: int) -> None:
        self._axis_number = value

    @property
    def reply_flag(self) -> str:
        """
        The reply flag indicates if the request was accepted (OK) or rejected (RJ).
        """

        return self._reply_flag

    @reply_flag.setter
    def reply_flag(self, value: str) -> None:
        self._reply_flag = value

    @property
    def status(self) -> str:
        """
        The device status contains BUSY when the axis is moving and IDLE otherwise.
        """

        return self._status

    @status.setter
    def status(self, value: str) -> None:
        self._status = value

    @property
    def warning_flag(self) -> str:
        """
        The warning flag contains the highest priority warning currently active for the device or axis.
        """

        return self._warning_flag

    @warning_flag.setter
    def warning_flag(self, value: str) -> None:
        self._warning_flag = value

    @property
    def data(self) -> str:
        """
        Response data which varies depending on the request.
        """

        return self._data

    @data.setter
    def data(self, value: str) -> None:
        self._data = value

    @property
    def message_type(self) -> MessageType:
        """
        Type of the reply received.
        """

        return self._message_type

    @message_type.setter
    def message_type(self, value: MessageType) -> None:
        self._message_type = value

    def __repr__(self) -> str:
        return str(self.__dict__)

    @staticmethod
    def from_protobuf(pb_data: main_pb2.GenericCommandResponse) -> 'Response':
        instance = Response()
        instance.device_address = pb_data.device_address
        instance.axis_number = pb_data.axis_number
        instance.reply_flag = pb_data.reply_flag
        instance.status = pb_data.status
        instance.warning_flag = pb_data.warning_flag
        instance.data = pb_data.data
        instance.message_type = MessageType(pb_data.message_type)
        return instance



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
# pylint: disable=W0201

from .protobufs import main_pb2


class FirmwareVersion:
    """
    Class representing version of firmware in the controller.
    """

    @property
    def major(self) -> int:
        """
        Major version number.
        """

        return self._major

    @major.setter
    def major(self, value: int) -> None:
        self._major = value

    @property
    def minor(self) -> int:
        """
        Minor version number.
        """

        return self._minor

    @minor.setter
    def minor(self, value: int) -> None:
        self._minor = value

    @property
    def build(self) -> int:
        """
        Build version number.
        """

        return self._build

    @build.setter
    def build(self, value: int) -> None:
        self._build = value

    def __repr__(self) -> str:
        return str(self.__dict__)

    @staticmethod
    def from_protobuf(pb_data: main_pb2.FirmwareVersion) -> 'FirmwareVersion':
        instance = FirmwareVersion()
        instance.major = pb_data.major
        instance.minor = pb_data.minor
        instance.build = pb_data.build
        return instance



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
# pylint: disable=W0201

from ..protobufs import main_pb2


class LockstepAxes:
    """
    The axis numbers of a lockstep group.
    """

    @property
    def axis_1(self) -> int:
        """
        The axis number used to set the first axis.
        """

        return self._axis_1

    @axis_1.setter
    def axis_1(self, value: int) -> None:
        self._axis_1 = value

    @property
    def axis_2(self) -> int:
        """
        The axis number used to set the second axis.
        """

        return self._axis_2

    @axis_2.setter
    def axis_2(self, value: int) -> None:
        self._axis_2 = value

    def __repr__(self) -> str:
        return str(self.__dict__)

    @staticmethod
    def from_protobuf(pb_data: main_pb2.LockstepAxes) -> 'LockstepAxes':
        instance = LockstepAxes()
        instance.axis_1 = pb_data.axis_1
        instance.axis_2 = pb_data.axis_2
        return instance



from .connection import Connection
from .device import Device
from .axis import Axis
from .all_axes import AllAxes
from .axis_settings import AxisSettings
from .device_settings import DeviceSettings
from .warnings import Warnings
from .warning_flags import WarningFlags
from .unknown_response_event import UnknownResponseEvent
from .device_identity import DeviceIdentity
from .device_io import DeviceIO
from .device_io_info import DeviceIOInfo
from .axis_identity import AxisIdentity
from .message_type import MessageType
from .axis_type import AxisType
from .alert_event import AlertEvent
from .lockstep_axes import LockstepAxes
from .lockstep import Lockstep
from .response import Response
from .setting_constants import SettingConstants



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

from typing import List
from .call import call, call_async
from .protobufs import main_pb2


class Tools:
    """
    Class providing various utility functions.
    """

    @staticmethod
    def list_serial_ports(
    ) -> List[str]:
        """
        Lists all serial ports on the computer.

        Returns:
            Array of serial port names.
        """
        request = main_pb2.EmptyRequest()
        response = main_pb2.ToolsListSerialPortsResponse()
        call("tools/list_serial_ports", request, response)
        return response.ports[:]

    @staticmethod
    async def list_serial_ports_async(
    ) -> List[str]:
        """
        Lists all serial ports on the computer.

        Returns:
            Array of serial port names.
        """
        request = main_pb2.EmptyRequest()
        response = main_pb2.ToolsListSerialPortsResponse()
        await call_async("tools/list_serial_ports", request, response)
        return response.ports[:]
